/**
* @file model_process.h
*
* Copyright (C) 2020. Huawei Technologies Co., Ltd. All rights reserved.
*
* This program is distributed in the hope that it will be useful,
* but WITHOUT ANY WARRANTY; without even the implied warranty of
* MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.
*/
#pragma once
#include <iostream>
#include "utils.h"
#include "acl/acl.h"
#include <vector>

struct DataInfo {
    void* data;
    size_t size;
};


class ModelProcess {
public:
    /**
    * @brief Constructor
    */
    ModelProcess();

    /**
    * @brief Destructor
    */
    virtual ~ModelProcess();


    /**
    * @brief load pcd.bin file
    * @param [in] file_path: model path
    * @param [in] num_point_dim: dimonsion of points
    * @param [in] buffer: contents of the file
    * @param [in] num_points: number of points
    * @return result
    */
    Result read_point(const std::string &file_path, const int num_point_dim,
                void **buffer, int *num_points);

    /**
    * @brief load model
    * @param [in] modelPath: model path
    * @return result
    */
    Result LoadModel(const char *modelPath);

    /**
    * @brief unload model
    */
    void UnloadModel();

    /**
    * @brief create model desc
    * @return result
    */
    Result CreateModelDesc();

    /**
    * @brief destroy desc
    */
    void DestroyModelDesc();


    /**
     * @brief Voxelize the space with fixed dimensions
     * @param [in] point_cloud_range_x_min: Minimum x-coordinate of the point cloud range
     * @param [in] point_cloud_range_y_min: Minimum y-coordinate of the point cloud range
     * @param [in] point_cloud_range_z_min: Minimum z-coordinate of the point cloud range
     * @param [in] voxel_size_x: Size of the voxel in the x-axis
     * @param [in] voxel_size_y: Size of the voxel in the y-axis
     * @param [in] voxel_size_z: Size of the voxel in the z-axis
     * @param [in] grid_size_x: Number of voxels in the x-axis
     * @param [in] grid_size_y: Number of voxels in the y-axis
     * @param [in] grid_size_z: Number of voxels in the z-axis
     * @param [in] max_num_points_in_voxel: Maximum number of points in a voxel
     * @param [in] max_voxels: Maximum number of voxels
     * @param [in] points: Pointer to the array of points
     * @param [in] num_point_dim: Dimension of each point (e.g., 3 for x, y, z)
     * @param [in] num_points: Total number of points
     * @param [in] voxels: Output array to store voxels
     * @param [in] coords: Array to store coordinates of voxels
     * @param [in] num_points_per_voxel: Array to store the number of points per voxel
     * @param [in] voxel_num: Variable to track the current voxel number
     * @return Result of the voxelization process
    */
    Result hard_voxelize(const float point_cloud_range_x_min,
                                 const float point_cloud_range_y_min,
                                 const float point_cloud_range_z_min,
                                 const float voxel_size_x, const float voxel_size_y,
                                 const float voxel_size_z, const int grid_size_x,
                                 const int grid_size_y, const int grid_size_z,
                                 const int max_num_points_in_voxel, const int max_voxels,
                                 const float* points, const int num_point_dim,
                                 const int num_points, float* voxels, int* coords,
                                 int* num_points_per_voxel, int* voxel_num);




    /**
    * @get input size by index
    * @param [in] index: input index
    * @param [out] inputSize: input size of index
    * @return result
    */
    Result GetInputSizeByIndex(const size_t index, size_t &inputSize);


    Result AddDatasetBuffer(aclmdlDataset *dataset, 
                                        void* buffer, uint32_t bufferSize);


    /**
    * @brief create model input
    * @param [in] inputDataBuffer: input buffer
    * @param [in] bufferSize: input buffer size
    * @return result
    */
    // Result CreateInput(void *inputDataBuffer, size_t bufferSize);
    Result CreateInput(std::vector<DataInfo>& inputData);
    /**
    * @brief destroy input resource
    */
    void DestroyInput();

    /**
    * @brief create output buffer
    * @return result
    */
    Result CreateOutput();

    /**
    * @brief destroy output resource
    */
    void DestroyOutput();

    /**
    * @brief model execute
    * @return result
    */
    Result Execute();

    /**
    * @brief dump model output result to file
    */
    void DumpModelOutputResult();

    /**
    * @brief print model output result
    */
    // void OutputModelResult();
    std::vector<float*> OutputModelResult();

private:
    uint32_t modelId_;
    size_t modelWorkSize_; // model work memory buffer size
    size_t modelWeightSize_; // model weight memory buffer size
    void *modelWorkPtr_; // model work memory buffer
    void *modelWeightPtr_; // model weight memory buffer
    bool loadFlag_;  // model load flag
    aclmdlDesc *modelDesc_;
    aclmdlDataset *input_;
    aclmdlDataset *output_;
};

